@@ -20,6 +20,8 @@
  */
 
 #include "gsimpleactiongroup.h"
+
+#include "gsimpleaction.h"
 #include "gaction.h"
 
 /**
@@ -148,9 +150,9 @@ g_simple_action_group_get_state (GActionGroup *group,
 }
 
 static void
-g_simple_action_group_set_state (GActionGroup *group,
-                                 const gchar  *action_name,
-                                 GVariant     *value)
+g_simple_action_group_change_state (GActionGroup *group,
+                                    const gchar  *action_name,
+                                    GVariant     *value)
 {
   GSimpleActionGroup *simple = G_SIMPLE_ACTION_GROUP (group);
   GAction *action;
@@ -160,7 +162,7 @@ g_simple_action_group_set_state (GActionGroup *group,
   if (action == NULL)
     return;
 
-  return g_action_set_state (action, value);
+  g_action_change_state (action, value);
 }
 
 static void
@@ -176,7 +178,7 @@ g_simple_action_group_activate (GActionGroup *group,
   if (action == NULL)
     return;
 
-  return g_action_activate (action, parameter);
+  g_action_activate (action, parameter);
 }
 
 static void
@@ -258,7 +260,7 @@ g_simple_action_group_iface_init (GActionGroupInterface *iface)
   iface->get_action_state_hint = g_simple_action_group_get_state_hint;
   iface->get_action_enabled = g_simple_action_group_get_enabled;
   iface->get_action_state = g_simple_action_group_get_state;
-  iface->change_action_state = g_simple_action_group_set_state;
+  iface->change_action_state = g_simple_action_group_change_state;
   iface->activate_action = g_simple_action_group_activate;
 }
 
@@ -378,3 +380,156 @@ g_simple_action_group_remove (GSimpleActionGroup *simple,
       g_hash_table_remove (simple->priv->table, action_name);
     }
 }
+
+/**
+ * GActionEntry:
+ * @name: the name of the action
+ * @activate: the callback to connect to the "activate" signal of the
+ *            action
+ * @parameter_type: the type of the parameter that must be passed to the
+ *                  activate function for this action, given as a single
+ *                  GVariant type string (or %NULL for no parameter)
+ * @state: the initial state for this action, given in GVariant text
+ *         format.  The state is parsed with no extra type information,
+ *         so type tags must be added to the string if they are
+ *         necessary.
+ * @change_state: the callback to connect to the "change-state" signal
+ *                of the action
+ *
+ * This struct defines a single action.  It is for use with
+ * g_simple_action_group_add_entries().
+ *
+ * The order of the items in the structure are intended to reflect
+ * frequency of use.  It is permissible to use an incomplete initialiser
+ * in order to leave some of the later values as %NULL.  All values
+ * after @name are optional.  Additional optional fields may be added in
+ * the future.
+ *
+ * See g_simple_action_group_add_entries() for an example.
+ **/
+
+/**
+ * g_simple_action_group_add_entries:
+ * @simple: a #GSimpleActionGroup
+ * @entries: a pointer to the first item in an array of #GActionEntry
+ *           structs
+ * @n_entries: the length of @entries, or -1
+ * @user_data: the user data for signal connections
+ *
+ * A convenience function for creating multiple #GSimpleAction instances
+ * and adding them to the action group.
+ *
+ * Each action is constructed as per one #GActionEntry.
+ *
+ * <example>
+ * <title>Using g_simple_action_group_add_entries()</title>
+ * <programlisting>
+ * static void
+ * activate_quit (GSimpleAction *simple,
+ *                GVariant      *parameter,
+ *                gpointer       user_data)
+ * {
+ *   exit (0);
+ * }
+ *
+ * static void
+ * activate_print_string (GSimpleAction *simple,
+ *                        GVariant      *parameter,
+ *                        gpointer       user_data)
+ * {
+ *   g_print ("%s\n", g_variant_get_string (parameter, NULL));
+ * }
+ *
+ * static GActionGroup *
+ * create_action_group (void)
+ * {
+ *   const GActionEntry entries[] = {
+ *     { "quit",         activate_quit              },
+ *     { "print-string", activate_print_string, "s" }
+ *   };
+ *   GSimpleActionGroup *group;
+ *
+ *   group = g_simple_action_group_new ();
+ *   g_simple_action_group_add_entries (group, entries, G_N_ELEMENTS (entries), NULL);
+ *
+ *   return G_ACTION_GROUP (group);
+ * }
+ * </programlisting>
+ * </example>
+ *
+ * Since: 2.30
+ **/
+void
+g_simple_action_group_add_entries (GSimpleActionGroup *simple,
+                                   const GActionEntry *entries,
+                                   gint                n_entries,
+                                   gpointer            user_data)
+{
+  gint i;
+
+  g_return_if_fail (G_IS_SIMPLE_ACTION_GROUP (simple));
+  g_return_if_fail (entries != NULL || n_entries == 0);
+
+  for (i = 0; n_entries == -1 ? entries[i].name != NULL : i < n_entries; i++)
+    {
+      const GActionEntry *entry = &entries[i];
+      const GVariantType *parameter_type;
+      GSimpleAction *action;
+
+      if (entry->parameter_type)
+        {
+          if (!g_variant_type_string_is_valid (entry->parameter_type))
+            {
+              g_critical ("g_simple_action_group_add_entries: the type "
+                          "string '%s' given as the parameter type for "
+                          "action '%s' is not a valid GVariant type "
+                          "string.  This action will not be added.",
+                          entry->parameter_type, entry->name);
+              return;
+            }
+
+          parameter_type = G_VARIANT_TYPE (entry->parameter_type);
+        }
+      else
+        parameter_type = NULL;
+
+      if (entry->state)
+        {
+          GError *error = NULL;
+          GVariant *state;
+
+          state = g_variant_parse (NULL, entry->state, NULL, NULL, &error);
+          if (state == NULL)
+            {
+              g_critical ("g_simple_action_group_add_entries: GVariant could "
+                          "not parse the state value given for action '%s' "
+                          "('%s'): %s.  This action will not be added.",
+                          entry->name, entry->state, error->message);
+              g_error_free (error);
+              continue;
+            }
+
+          action = g_simple_action_new_stateful (entry->name,
+                                                 parameter_type,
+                                                 state);
+
+          g_variant_unref (state);
+        }
+      else
+        {
+          action = g_simple_action_new (entry->name,
+                                        parameter_type);
+        }
+
+      if (entry->activate != NULL)
+        g_signal_connect (action, "activate",
+                          G_CALLBACK (entry->activate), user_data);
+
+      if (entry->change_state != NULL)
+        g_signal_connect (action, "change-state",
+                          G_CALLBACK (entry->change_state), user_data);
+
+      g_simple_action_group_insert (simple, G_ACTION (action));
+      g_object_unref (action);
+    }
+}
