@@ -52,11 +52,10 @@
 #include "gstdio.h"
 #include "gstrfuncs.h"
 #include "gatomic.h"
-#include "gbuffer.h"
+#include "gbufferprivate.h"
 
 #include "glibintl.h"
 
-#include "galias.h"
 
 #ifndef _O_BINARY
 #define _O_BINARY 0
@@ -113,14 +112,19 @@ g_mapped_file_destroy (GMappedFile *file)
  * Maps a file into memory. On UNIX, this is using the mmap() function.
  *
  * If @writable is %TRUE, the mapped buffer may be modified, otherwise
- * it is an error to modify the mapped buffer. Modifications to the buffer 
- * are not visible to other processes mapping the same file, and are not 
+ * it is an error to modify the mapped buffer. Modifications to the buffer
+ * are not visible to other processes mapping the same file, and are not
  * written back to the file.
  *
  * Note that modifications of the underlying file might affect the contents
- * of the #GMappedFile. Therefore, mapping should only be used if the file 
+ * of the #GMappedFile. Therefore, mapping should only be used if the file
  * will not be modified, or if all modifications of the file are done
- * atomically (e.g. using g_file_set_contents()). 
+ * atomically (e.g. using g_file_set_contents()).
+ *
+ * If @filename is the name of an empty, regular file, the function
+ * will successfully return an empty #GMappedFile. In other cases of
+ * size 0 (e.g. device files such as /dev/null), @error will be set
+ * to the #GFileError value #G_FILE_ERROR_INVAL.
  *
  * Return value: a newly allocated #GMappedFile which must be unref'd
  *    with g_mapped_file_unref(), or %NULL if the mapping failed.
@@ -174,7 +178,11 @@ g_mapped_file_new (const gchar  *filename,
       goto out;
     }
 
-  if (st.st_size == 0)
+  /* mmap() on size 0 will fail with EINVAL, so we avoid calling mmap()
+   * in that case -- but only if we have a regular file; we still want
+   * attempts to mmap a character device to fail, for example.
+   */
+  if (st.st_size == 0 && S_ISREG (st.st_mode))
     {
       file->length = 0;
       file->contents = NULL;
@@ -316,7 +324,6 @@ GMappedFile *
 g_mapped_file_ref (GMappedFile *file)
 {
   g_return_val_if_fail (file != NULL, NULL);
-  g_return_val_if_fail (file->ref_count > 0, file);
 
   g_atomic_int_inc (&file->ref_count);
 
@@ -338,11 +345,7 @@ void
 g_mapped_file_unref (GMappedFile *file)
 {
   g_return_if_fail (file != NULL);
-  g_return_if_fail (file->ref_count > 0);
 
   if (g_atomic_int_dec_and_test (&file->ref_count))
     g_mapped_file_destroy (file);
 }
-
-#define __G_MAPPED_FILE_C__
-#include "galiasdef.c"
