@@ -105,6 +105,7 @@
 
 #include "gbinding.h"
 #include "genums.h"
+#include "gmarshal.h"
 #include "gobject.h"
 #include "gsignal.h"
 #include "gparamspecs.h"
@@ -124,6 +125,7 @@ g_binding_flags_get_type (void)
         { G_BINDING_DEFAULT, "G_BINDING_DEFAULT", "default" },
         { G_BINDING_BIDIRECTIONAL, "G_BINDING_BIDIRECTIONAL", "bidirectional" },
         { G_BINDING_SYNC_CREATE, "G_BINDING_SYNC_CREATE", "sync-create" },
+        { G_BINDING_INVERT_BOOLEAN, "G_BINDING_INVERT_BOOLEAN", "invert-boolean" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -149,8 +151,8 @@ struct _GBinding
   GObject *target;
 
   /* the property names are interned, so they should not be freed */
-  gchar *source_property;
-  gchar *target_property;
+  const gchar *source_property;
+  const gchar *target_property;
 
   GParamSpec *source_pspec;
   GParamSpec *target_pspec;
@@ -301,21 +303,44 @@ done:
   return TRUE;
 }
 
+static inline gboolean
+default_invert_boolean_transform (const GValue *value_a,
+                                  GValue       *value_b)
+{
+  gboolean value;
+
+  g_assert (G_VALUE_HOLDS_BOOLEAN (value_a));
+  g_assert (G_VALUE_HOLDS_BOOLEAN (value_b));
+
+  value = g_value_get_boolean (value_a);
+  value = !value;
+
+  g_value_set_boolean (value_b, value);
+
+  return TRUE;
+}
+
 static gboolean
-default_transform_to (GBinding     *binding G_GNUC_UNUSED,
+default_transform_to (GBinding     *binding,
                       const GValue *value_a,
                       GValue       *value_b,
                       gpointer      user_data G_GNUC_UNUSED)
 {
+  if (binding->flags & G_BINDING_INVERT_BOOLEAN)
+    return default_invert_boolean_transform (value_a, value_b);
+
   return default_transform (value_a, value_b);
 }
 
 static gboolean
-default_transform_from (GBinding     *binding G_GNUC_UNUSED,
+default_transform_from (GBinding     *binding,
                         const GValue *value_a,
                         GValue       *value_b,
                         gpointer      user_data G_GNUC_UNUSED)
 {
+  if (binding->flags & G_BINDING_INVERT_BOOLEAN)
+    return default_invert_boolean_transform (value_a, value_b);
+
   return default_transform (value_a, value_b);
 }
 
@@ -717,7 +742,7 @@ g_binding_get_target (GBinding *binding)
  *
  * Since: 2.26
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_binding_get_source_property (GBinding *binding)
 {
   g_return_val_if_fail (G_IS_BINDING (binding), NULL);
@@ -736,7 +761,7 @@ g_binding_get_source_property (GBinding *binding)
  *
  * Since: 2.26
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_binding_get_target_property (GBinding *binding)
 {
   g_return_val_if_fail (G_IS_BINDING (binding), NULL);
@@ -778,6 +803,12 @@ g_binding_get_target_property (GBinding *binding)
  *
  * A #GObject can have multiple bindings.
  *
+ * <note>The same @user_data parameter will be used for both @transform_to
+ * and @transform_from transformation functions; the @notify function will
+ * be called once, when the binding is removed. If you need different data
+ * for each transformation function, please use
+ * g_object_bind_property_with_closures() instead.</note>
+ *
  * Return value: (transfer none): the #GBinding instance representing the
  *   binding between the two #GObject instances. The binding is released
  *   whenever the #GBinding reference count reaches zero.
@@ -809,11 +840,14 @@ g_object_bind_property_full (gpointer               source,
       return NULL;
     }
 
-  if (transform_to == NULL)
-    transform_to = default_transform_to;
-
-  if (transform_from == NULL)
-    transform_from = default_transform_from;
+  /* remove the G_BINDING_INVERT_BOOLEAN flag in case we have
+   * custom transformation functions
+   */
+  if ((flags & G_BINDING_INVERT_BOOLEAN) &&
+      (transform_to != NULL || transform_from != NULL))
+    {
+      flags &= ~G_BINDING_INVERT_BOOLEAN;
+    }
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (source), source_property);
   if (pspec == NULL)
@@ -844,6 +878,18 @@ g_object_bind_property_full (gpointer               source,
       return NULL;
     }
 
+  if ((flags & G_BINDING_INVERT_BOOLEAN) &&
+      !(G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_BOOLEAN))
+    {
+      g_warning ("%s: The G_BINDING_INVERT_BOOLEAN flag can only be used "
+                 "when binding boolean properties; the source property '%s' "
+                 "is of type '%s'",
+                 G_STRLOC,
+                 source_property,
+                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+      return NULL;
+    }
+
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (target), target_property);
   if (pspec == NULL)
     {
@@ -866,13 +912,25 @@ g_object_bind_property_full (gpointer               source,
   if ((flags & G_BINDING_BIDIRECTIONAL) &&
       !(pspec->flags & G_PARAM_READABLE))
     {
-      g_warning ("%s: The starget object of type %s has no writable property called '%s'",
+      g_warning ("%s: The target object of type %s has no readable property called '%s'",
                  G_STRLOC,
                  G_OBJECT_TYPE_NAME (target),
                  target_property);
       return NULL;
     }
 
+  if ((flags & G_BINDING_INVERT_BOOLEAN) &&
+      !(G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_BOOLEAN))
+    {
+      g_warning ("%s: The G_BINDING_INVERT_BOOLEAN flag can only be used "
+                 "when binding boolean properties; the target property '%s' "
+                 "is of type '%s'",
+                 G_STRLOC,
+                 target_property,
+                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+      return NULL;
+    }
+
   binding = g_object_new (G_TYPE_BINDING,
                           "source", source,
                           "source-property", source_property,
@@ -881,9 +939,12 @@ g_object_bind_property_full (gpointer               source,
                           "flags", flags,
                           NULL);
 
-  /* making these properties would be awkward, though not impossible */
-  binding->transform_s2t = transform_to;
-  binding->transform_t2s = transform_from;
+  if (transform_to != NULL)
+    binding->transform_s2t = transform_to;
+
+  if (transform_from != NULL)
+    binding->transform_t2s = transform_from;
+
   binding->transform_data = user_data;
   binding->notify = notify;
 
@@ -951,3 +1012,177 @@ g_object_bind_property (gpointer       source,
                                       NULL,
                                       NULL, NULL);
 }
+
+typedef struct _TransformData
+{
+  GClosure *transform_to_closure;
+  GClosure *transform_from_closure;
+} TransformData;
+
+static gboolean
+bind_with_closures_transform_to (GBinding     *binding,
+                                 const GValue *source,
+                                 GValue       *target,
+                                 gpointer      data)
+{
+  TransformData *t_data = data;
+  GValue params[3] = { { 0, }, { 0, }, { 0, } };
+  GValue retval = { 0, };
+  gboolean res;
+
+  g_value_init (&params[0], G_TYPE_BINDING);
+  g_value_set_object (&params[0], binding);
+
+  g_value_init (&params[1], G_TYPE_VALUE);
+  g_value_set_boxed (&params[1], source);
+
+  g_value_init (&params[2], G_TYPE_VALUE);
+  g_value_set_boxed (&params[2], target);
+
+  g_value_init (&retval, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&retval, FALSE);
+
+  g_closure_invoke (t_data->transform_to_closure, &retval, 3, params, NULL);
+
+  res = g_value_get_boolean (&retval);
+  if (res)
+    {
+      const GValue *out_value = g_value_get_boxed (&params[2]);
+
+      g_assert (out_value != NULL);
+
+      g_value_copy (out_value, target);
+    }
+
+  g_value_unset (&params[0]);
+  g_value_unset (&params[1]);
+  g_value_unset (&params[2]);
+  g_value_unset (&retval);
+
+  return res;
+}
+
+static gboolean
+bind_with_closures_transform_from (GBinding     *binding,
+                                   const GValue *source,
+                                   GValue       *target,
+                                   gpointer      data)
+{
+  TransformData *t_data = data;
+  GValue params[3] = { { 0, }, { 0, }, { 0, } };
+  GValue retval = { 0, };
+  gboolean res;
+
+  g_value_init (&params[0], G_TYPE_BINDING);
+  g_value_set_object (&params[0], binding);
+
+  g_value_init (&params[1], G_TYPE_VALUE);
+  g_value_set_boxed (&params[1], source);
+
+  g_value_init (&params[2], G_TYPE_VALUE);
+  g_value_set_boxed (&params[2], target);
+
+  g_value_init (&retval, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&retval, FALSE);
+
+  g_closure_invoke (t_data->transform_from_closure, &retval, 3, params, NULL);
+
+  res = g_value_get_boolean (&retval);
+  if (res)
+    {
+      const GValue *out_value = g_value_get_boxed (&params[2]);
+
+      g_assert (out_value != NULL);
+
+      g_value_copy (out_value, target);
+    }
+
+  g_value_unset (&params[0]);
+  g_value_unset (&params[1]);
+  g_value_unset (&params[2]);
+  g_value_unset (&retval);
+
+  return res;
+}
+
+static void
+bind_with_closures_free_func (gpointer data)
+{
+  TransformData *t_data = data;
+
+  if (t_data->transform_to_closure != NULL)
+    g_closure_unref (t_data->transform_to_closure);
+
+  if (t_data->transform_from_closure != NULL)
+    g_closure_unref (t_data->transform_from_closure);
+
+  g_slice_free (TransformData, t_data);
+}
+
+/**
+ * g_object_bind_property_with_closures:
+ * @source: the source #GObject
+ * @source_property: the property on @source to bind
+ * @target: the target #GObject
+ * @target_property: the property on @target to bind
+ * @flags: flags to pass to #GBinding
+ * @transform_to: a #GClosure wrapping the transformation function
+ *   from the @source to the @target, or %NULL to use the default
+ * @transform_from: a #GClosure wrapping the transformation function
+ *   from the @target to the @source, or %NULL to use the default
+ *
+ * Creates a binding between @source_property on @source and @target_property
+ * on @target, allowing you to set the transformation functions to be used by
+ * the binding.
+ *
+ * This function is the language bindings friendly version of
+ * g_object_bind_property_full(), using #GClosure<!-- -->s instead of
+ * function pointers.
+ *
+ * Rename to: g_object_bind_property_full
+ *
+ * Return value: (transfer none): the #GBinding instance representing the
+ *   binding between the two #GObject instances. The binding is released
+ *   whenever the #GBinding reference count reaches zero.
+ *
+ * Since: 2.26
+ */
+GBinding *
+g_object_bind_property_with_closures (gpointer       source,
+                                      const gchar   *source_property,
+                                      gpointer       target,
+                                      const gchar   *target_property,
+                                      GBindingFlags  flags,
+                                      GClosure      *transform_to,
+                                      GClosure      *transform_from)
+{
+  TransformData *data;
+
+  data = g_slice_new0 (TransformData);
+
+  if (transform_to != NULL)
+    {
+      if (G_CLOSURE_NEEDS_MARSHAL (transform_to))
+        g_closure_set_marshal (transform_to, g_cclosure_marshal_BOOLEAN__BOXED_BOXED);
+
+      data->transform_to_closure = g_closure_ref (transform_to);
+      g_closure_sink (data->transform_to_closure);
+    }
+
+  if (transform_from != NULL)
+    {
+      if (G_CLOSURE_NEEDS_MARSHAL (transform_from))
+        g_closure_set_marshal (transform_from, g_cclosure_marshal_BOOLEAN__BOXED_BOXED);
+
+      data->transform_from_closure = g_closure_ref (transform_from);
+      g_closure_sink (data->transform_from_closure);
+    }
+
+  return g_object_bind_property_full (source, source_property,
+                                      target, target_property,
+                                      flags,
+                                      transform_to != NULL ? bind_with_closures_transform_to : NULL,
+                                      transform_from != NULL ? bind_with_closures_transform_from : NULL,
+                                      data,
+                                      bind_with_closures_free_func);
+}
